@@ -100,7 +100,7 @@ npm whoami
 echo "Making sure we are logged in to crates.io..."
 cargo owner --list fend >/dev/null
 
-PATH="$HOME/.cargo/bin:$PATH"
+PATH="${XDG_DATA_HOME-$HOME/.local/share}/cargo/bin:$PATH"
 echo "Ensure that we are using Rustup"
 rustc_cmd=$(command -v rustc)
 if [[ ! "$rustc_cmd" =~ .(cargo|rustup)/bin/rustc$ ]]; then
@@ -209,8 +209,6 @@ zip --junk-paths "$TMPDIR/artifacts/fend-$NEW_VERSION-linux-x86_64-gnu.zip" \
 	"$TMPDIR/artifacts/fend-$NEW_VERSION-linux-x86_64-gnu/fend"
 zip --junk-paths "$TMPDIR/artifacts/fend-$NEW_VERSION-linux-aarch64-gnu.zip" \
 	"$TMPDIR/artifacts/fend-$NEW_VERSION-linux-aarch64-gnu/fend"
-zip --junk-paths "$TMPDIR/artifacts/fend-$NEW_VERSION-linux-armv7-gnueabihf.zip" \
-	"$TMPDIR/artifacts/fend-$NEW_VERSION-linux-armv7-gnueabihf/fend"
 zip --junk-paths "$TMPDIR/artifacts/fend-$NEW_VERSION-linux-x86_64-musl.zip" \
 	"$TMPDIR/artifacts/fend-$NEW_VERSION-linux-x86_64-musl/fend"
 zip --junk-paths "$TMPDIR/artifacts/fend-$NEW_VERSION-macos-aarch64.zip" \
@@ -230,7 +228,6 @@ gh release --repo printfn/fend \
 	--draft \
 	"$TMPDIR/artifacts/fend-$NEW_VERSION-linux-x86_64-gnu.zip" \
 	"$TMPDIR/artifacts/fend-$NEW_VERSION-linux-aarch64-gnu.zip" \
-	"$TMPDIR/artifacts/fend-$NEW_VERSION-linux-armv7-gnueabihf.zip" \
 	"$TMPDIR/artifacts/fend-$NEW_VERSION-linux-x86_64-musl.zip" \
 	"$TMPDIR/artifacts/fend-$NEW_VERSION-macos-aarch64.zip" \
 	"$TMPDIR/artifacts/fend-$NEW_VERSION-windows-x64-exe.zip" \
